-- 
-- ShamWow_Enhance

ShamWow_Enhance = LibStub( "AceAddon-3.0" ):NewAddon( "ShamWow_Enhance", "AceConsole-3.0", "AceEvent-3.0" );
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local mainFrame, cooldownFrame, cooldownFrame1, cooldownFrame2, target, timeText;
local earthshieldTarget;
local ENHANCE_MODE = 1;
local RESTO_MODE = 2;
local ELEM_MODE = 3;
local AUTO_MODE = 4;
local mode = 0;
local lastFlameShockCast = 0;

local function swPrint(s)
  DEFAULT_CHAT_FRAME:AddMessage("ShamWow_Enhance: ".. tostring(s));
end

-- spell info
local stormstrikeId = 17364;
local lavalashId = 60103;
local earthshockId = 25454;
local maelstromId = 51532;
local windfuryId = 25505;
local flametongueId = 25489;
local lightningBoltId = 25449;
local watershieldId = 33736;
local lightningshieldId = 25472;
local earthshieldId = 49284;
local healingWaveId = 49273;
local shamanisticRageId = 30823;
local clearcastId = 16246;
local feralSpiritId = 51533;
local riptideId = 61299;
local earthshieldId = 32593;
local earthlivingId = 51730;
local lesserHealingWaveId = 8004;
local naturesSwiftnessId = 17116;
local manatideId = 16190;
local flameshockId = 8050;
local lavaburstId = 51505;
local chainlightningId = 25442;
local totemOfWrathId = 57720;
local elementalMasteryId = 16166;
local thunderstormId = 51490;
local thunderstormName = GetSpellInfo(thunderstormId);
local elementalMasteryName = GetSpellInfo(elementalMasteryId);
local totemOfWrathName = GetSpellInfo(totemOfWrathId);
local flameshockName = GetSpellInfo(flameshockId);
local lavaburstName = GetSpellInfo(lavaburstId);
local manatideName = GetSpellInfo(manatideId);
local chainlightningName = GetSpellInfo(chainlightningId);
local naturesSwiftnessName = GetSpellInfo(naturesSwiftnessId);
local lesserHealingWaveName = GetSpellInfo(lesserHealingWaveId);
local riptideName = GetSpellInfo(riptideId);
local earthshieldName = GetSpellInfo(earthshieldId);
local earthlivingName = GetSpellInfo(51730);
local clearcastName = GetSpellInfo(clearcastId);
local feralSpiritName = GetSpellInfo(feralSpiritId);
local autoattackName = "Attack"; -- fixme does this have an id?
local stormstrikeName = GetSpellInfo(stormstrikeId);
local lavalashName = GetSpellInfo(lavalashId);
local earthshockName = GetSpellInfo(earthshockId);
local maelstromName = GetSpellInfo(maelstromId);
local lightningBoltName = GetSpellInfo(lightningBoltId);
local windfuryName = GetSpellInfo(windfuryId);
local flametongueName = GetSpellInfo(flametongueId);
local watershieldName = GetSpellInfo(watershieldId);
local lightningshieldName = GetSpellInfo(lightningshieldId);
local earthshieldName = GetSpellInfo(earthshieldId);
local cooldownSpellName = lightningBoltName;
local healingWaveName = GetSpellInfo(healingWaveId);
local shamanisticRageName = GetSpellInfo(shamanisticRageId);

local spellQueue = {};
local maxQueueSize = 4;
local groupCount = 0;
local hostile = false;
local timeLeft = 0;
local inCombat = false;

-- configs
local defaults = {
 char = {
   x = 0,
   y = 0,
   relativePoint = "CENTER",
   mode = 4,
   size = 50,
	 spacing = 15,
   locked = false,
   threatThreshold = 70,
   updateFrequency = .2
 }
};

-- random variables
local playerId;

local function canCastSpell(spellName, time)
  local startTime, duration = GetSpellCooldown(spellName);
  if duration then
    local nextCastTime = startTime + duration;
    return nextCastTime <= time;
  end
end

function soonestCooldown(...)
  local castInfo = {};
  for i=1,select('#',...) do
    local spellName = select(i, ...);
    local startTime, duration = GetSpellCooldown(spellName);
    if duration then
      local nextCastTime = startTime + duration;
      if castInfo.castTime == nil or castInfo.castTime > nextCastTime then
        castInfo.castTime = nextCastTime;
        castInfo.spell = spellName;
      end
    end
  end
  return castInfo;
end

function ShamWow_Enhance:SaveLocation()
  point, relativeTo, relativePoint, xOfs, yOfs = mainFrame:GetPoint();
  self.db.char.x = xOfs;
  self.db.char.y = yOfs;
  self.db.char.relativePoint = relativePoint;
end

local function RepositionFrames()
  for i=maxQueueSize,1,-1 do
		local spacing = ShamWow_Enhance.db.char.spacing;
		local f = spellQueueFrames[i];
    f:SetWidth(ShamWow_Enhance.db.char.size)
    f:SetHeight(ShamWow_Enhance.db.char.size)
    if i == maxQueueSize then
      f:SetPoint("CENTER", spacing * (maxQueueSize-1), 0);
    else
		  f:SetPoint("CENTER", -spacing, 0);
    end
  end
end

function ShamWow_Enhance:OnInitialize()
  local AceConfigReg = LibStub("AceConfigRegistry-3.0")
  local AceConfigDialog = LibStub("AceConfigDialog-3.0")

  self.db = LibStub("AceDB-3.0"):New("ShamWow_EnhanceDB", defaults, "char")
  LibStub("AceConfig-3.0"):RegisterOptionsTable("ShamWow_Enhance", self:GetOptions(), {"ShamWow_Enhance", "swe"} )
  self.optionsFrame = AceConfigDialog:AddToBlizOptions("ShamWow_Enhance","ShamWow_Enhance")
  self.db:RegisterDefaults(defaults);

  mainFrame = CreateFrame("Frame","ShamWow_EnhanceDisplayFrame",UIParent)
  mainFrame:SetFrameStrata("BACKGROUND")
  mainFrame:SetWidth(self.db.char.size)
  mainFrame:SetHeight(self.db.char.size)

  mainFrame:EnableMouse(true)
  mainFrame:SetMovable(true)
  mainFrame:SetClampedToScreen(true)
  mainFrame:SetScript("OnMouseDown",
    function(self)
      if not ShamWow_Enhance.db.char.locked then
        self:StartMoving();
      end
    end);
  mainFrame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing(); ShamWow_Enhance:SaveLocation(); end);
  mainFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing(); ShamWow_Enhance:SaveLocation(); end)
  mainFrame:ClearAllPoints();
  mainFrame:SetPoint(self.db.char.relativePoint, self.db.char.x, self.db.char.y);

  spellQueueFrames = {};
  for i=maxQueueSize,1,-1 do
    local parentFrame;
    if i == maxQueueSize then
      parentFrame = mainFrame;
    else
      parentFrame = spellQueueFrames[i+1];
    end
    local f = CreateFrame("Frame","ShamWow_EnhanceDisplayFrame" .. i, parentFrame)
    f:SetFrameStrata("BACKGROUND")
    f:SetWidth(self.db.char.size)
    f:SetHeight(self.db.char.size)
    f:EnableMouse(false)
    f:SetMovable(false)
    f:SetClampedToScreen(true)
    f:ClearAllPoints();
		local spacing = ShamWow_Enhance.db.char.spacing;
    --if i == maxQueueSize then
    --  f:SetPoint("CENTER", spacing * (maxQueueSize-1), 0);
    --else
    --  f:SetPoint("CENTER", -spacing, 0);
    --end
    f.spellTexture = f:CreateTexture(nil,"BACKGROUND");
    f.spellTexture:SetAllPoints(f);
    f.overlayTexture = f:CreateTexture(nil,"OVERLAY");
    f.overlayTexture:SetAllPoints(f);
    spellQueueFrames[i] = f;
  end
  cooldownFrame1 = CreateFrame("Frame","ShamWow_ClearCastFrame", spellQueueFrames[1])
  cooldownFrame1:SetFrameStrata("BACKGROUND")
  local ccSize = self.db.char.size / 2;
  cooldownFrame1:SetWidth(ccSize);
  cooldownFrame1:SetHeight(ccSize);
  cooldownFrame1:SetPoint("TOPLEFT", -ccSize, 0);
  cooldownFrame1:EnableMouse(false)
  cooldownFrame1:SetMovable(false)
  cooldownFrame1:SetClampedToScreen(true)
  cooldownFrame1.texture = cooldownFrame1:CreateTexture(nil,"OVERLAY");
  cooldownFrame1.texture:SetAllPoints(cooldownFrame1);

  cooldownFrame2 = CreateFrame("Frame","ShamWow_FeralSpiritFrame", spellQueueFrames[1])
  cooldownFrame2:SetFrameStrata("BACKGROUND")
  cooldownFrame2:SetWidth(ccSize);
  cooldownFrame2:SetHeight(ccSize);
  cooldownFrame2:SetPoint("TOPLEFT", -ccSize, -ccSize);
  cooldownFrame2:EnableMouse(false)
  cooldownFrame2:SetMovable(false)
  cooldownFrame2:SetClampedToScreen(true)
  cooldownFrame2.texture = cooldownFrame2:CreateTexture(nil,"OVERLAY");
  cooldownFrame2.texture:SetAllPoints(cooldownFrame2);

  cooldownFrame = CreateFrame("Cooldown","$parent_cooldown", spellQueueFrames[1]);
  cooldownFrame:SetAllPoints(mainFrame);

  local testfont = media:Fetch("font", 'Arial');

  if not mainFrame.text then
    mainFrame.text = mainFrame:CreateFontString(nil,"OVERLAY")
  end
  mainFrame.text:SetFont(testfont, 12, "THICKOUTLINE");
  mainFrame.text:ClearAllPoints()
  mainFrame.text:SetTextColor(.8, 0, 0, 1)
  mainFrame.text:SetPoint("BOTTOM", mainFrame, "BOTTOM", 0, -13);
  mainFrame.text:SetText("")

  if not mainFrame.alertText then
    mainFrame.alertText = mainFrame:CreateFontString(nil,"OVERLAY")
  end
  mainFrame.alertText:SetFont(testfont, 12, "THICKOUTLINE");
  mainFrame.alertText:ClearAllPoints()
  mainFrame.alertText:SetTextColor(1, 1, 0, 1)
  mainFrame.alertText:SetPoint("BOTTOM", mainFrame, "TOP", 5, 2);
  mainFrame.alertText:SetText("")

  timeText = spellQueueFrames[1]:CreateFontString(nil,"OVERLAY")
  timeText:SetFont(testfont, 12, "THICKOUTLINE");
  timeText:ClearAllPoints()
  timeText:SetTextColor(0, 1, 0, 1)
  timeText:SetPoint("CENTER", mainFrame, "CENTER", 0, 0);
  timeText:SetText("")

	RepositionFrames();

  --mainFrame.spellTexture = mainFrame:CreateTexture(nil,"BACKGROUND");
  --mainFrame.spellTexture:SetAllPoints(mainFrame);
	swPrint('Initialized');
end

local function AlertCheck()
  if (not hostile) or groupCount == 0 or ShamWow_Enhance.db.char.threatThreshold > 100 then
    mainFrame.text:SetText("");
  else
    local _,_,threatpct,_,_ = UnitDetailedThreatSituation("player", "target");
    if not threatpct or threatpct < ShamWow_Enhance.db.char.threatThreshold then
      mainFrame.text:SetText("");
    else
      mainFrame.text:SetText(string.format("%.0f", threatpct) .. '%');
    end
  end
  local nastySpellName = ""; -- for tracking nasty spell reflect debuffs
  if (hostile) then
    -- check target buffs
    for i=1,40 do
      local name = UnitBuff("target", i);
      if not name then
        break; -- end of buffs
      end
      if name:lower():find('reflect') then
        nastySpellName = name;
        break;
      end
    end
  end
  mainFrame.alertText:SetText(nastySpellName);
end

-- this colors the overlay frame based on range, mana
local function FrameStatusCheck(f, spell)
  if spell == autoattackName or IsSpellInRange(spell, 'target') == 0 then
    f.overlayTexture:SetTexture(1, 0, 0, .5);
  else
    name, _, _, cost = GetSpellInfo(spell);
    if cost and UnitPower('player') < cost then
      f.overlayTexture:SetTexture(0, 0, 1, .5);
    else
      f.overlayTexture:SetTexture(nil);
    end
  end
end

function ShamWow_Enhance:PARTY_MEMBERS_CHANGED(...)
  groupCount = GetNumRaidMembers();
  if groupCount == 0 then
    groupType = "party";
    groupCount = GetNumPartyMembers();
  else
    groupType = "raid";
    groupCount = groupCount - 1; -- don't include yourself
  end
end

local function addSpell(spellName)
  spellQueue[#spellQueue+1] = spellName;
end

local function HasEarthShield(unitName)
  for i=1,40 do
    local name, _, _, _, _, _, _, isMine = UnitBuff(unitName, i);
    if not name then
      break; -- end of buffs
    end
    if isMine and name == earthshieldName then
      return true;
    end
  end
  return false;
end

local function RecalcRestoQueue(nextCastTime)
  local hasMainHandEnchant = GetWeaponEnchantInfo();
  if not hasMainHandEnchant then
    addSpell(earthlivingName);
  end

  -- check buffs
  local ccTexture = nil;
  local hasShield = false;
  local maelstrom = false;
  local earthshieldFound = false;
  for i=1,40 do
    local name, _, _, _, _, _, _, isMine = UnitBuff("player", i);
    if not name then
      break; -- end of buffs
    end
    if name == watershieldName or name == lightningshieldName then
      hasShield = true;
    elseif name == earthshieldName then
      hasShield = true;
      earthshieldFound = true;
    elseif name == clearcastName then
      ccTexture = texture;
    end
  end

  if groupCount > 0 then
    -- check for earthshield
    if (not earthshieldFound) and earthshieldTarget and HasEarthShield(earthshieldTarget) then
      earthshieldFound = true;
    end
    -- scan the party/raid for earthshield
    if not earthshieldFound then
      for groupIndex = 1,groupCount do
        local unitName = groupType .. groupIndex;
        if not UnitExists(unitName) then
          break;
        end 
        if HasEarthShield(unitName) then
          earthshieldFound = true;
          earthshieldTarget = unitName;
          break;
        end
      end
    end
		if (not earthshieldFound) then
	    addSpell(earthshieldName);
	  end
  end

  if canCastSpell(manatideName, nextCastTime) then
    cooldownFrame1.texture:SetTexture(GetSpellTexture(manatideName));
  else
    cooldownFrame1.texture:SetTexture(nil);
  end

  if (not hasShield) then
    addSpell(watershieldName);
  end

  if canCastSpell(naturesSwiftnessName, nextCastTime) then
    cooldownFrame2.texture:SetTexture(GetSpellTexture(naturesSwiftnessName));
  else
    cooldownFrame2.texture:SetTexture(nil);
  end

  if canCastSpell(riptideName, nextCastTime) then
    addSpell(riptideName);
  end
  addSpell(lesserHealingWaveName);

end

local function RecalcElemQueue(nextCastTime)
  local hasMainHandEnchant = GetWeaponEnchantInfo();
  if not hasMainHandEnchant then
    addSpell(flametongueName);
  end

  -- check buffs
  local ccTexture = nil;
  local hasShield = false;
  local maelstrom = false;
  local earthshieldFound = false;
  for i=1,40 do
    local name, _, _, _, _, _, _, isMine = UnitBuff("player", i);
    if not name then
      break; -- end of buffs
    end
    if name == watershieldName or name == lightningshieldName or name == earthshieldName then
      hasShield = true;
    end
  end

  if (inCombat or hostile) and canCastSpell(elementalMasteryName, nextCastTime) then
    cooldownFrame1.texture:SetTexture(GetSpellTexture(elementalMasteryName));
	else
    cooldownFrame1.texture:SetTexture(nil);
  end

  if (inCombat or hostile) and canCastSpell(thunderstormName, nextCastTime) then
    cooldownFrame2.texture:SetTexture(GetSpellTexture(thunderstormName));
  else
    cooldownFrame2.texture:SetTexture(nil);
  end

  if (not hasShield) then
    addSpell(watershieldName);
  end

	if hostile then
  	local fsTimeLeft = 0;
    -- check if flameshock has already been applied
    for i=1,40 do
      local name, _, _, _, _, _, expirationTime, isMine = UnitDebuff("target", i);
      if not name then
        break; -- end of debuffs
      end
     if isMine and name == flameshockName then
       fsTimeLeft = expirationTime - nextCastTime;
  		 break;
     end
    end
  
  	local currentSpell = UnitCastingInfo('player');
		local flameshockReady = canCastSpell(flameshockName, nextCastTime);
		local lavaburstReady = canCastSpell(lavaburstName, nextCastTime);
		if currentSpell == lavaburstName then
		  lavaburstReady = false;
		end
		if lavaburstReady then
		  if fsTimeLeft >= 1.5 then
				addSpell(lavaburstName);
				addSpell(lightningBoltName);
			elseif flameshockReady then
				addSpell(flameshockName);
				addSpell(lavaburstName);
			else
				addSpell(lightningBoltName);
			end
		elseif fsTimeLeft <= 0 then
		  addSpell(flameshockName);
			addSpell(lightningBoltName);
		else
		  addSpell(lightningBoltName);
		end
	end
end

local function RecalcEnhanceQueue(nextCastTime)
  local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo();
  if not hasMainHandEnchant then
    addSpell(windfuryName);
  end

  if (not hasOffHandEnchant) and OffhandHasWeapon() then
    addSpell(flametongueName);
  end

  -- check target debuffs
  local hasSSDebuff = false;
  for i=1,40 do
    local name = UnitDebuff("target", i);
    if not name then
      break; -- end of debuffs
    end
    if name == stormstrikeName then
      hasSSDebuff = true;
    end
  end

  -- check buffs
  local ccTexture = nil;
  local hasShield = false;
  local maelstrom = false;
  for i=1,40 do
    local name, _, texture, count = UnitBuff("player", i);
    if not name then
      break; -- end of buffs
    end
    if name == maelstromName then
      if hostile and count == 5 then
        maelstrom = true;
      end
    elseif name == watershieldName or name == lightningshieldName or name == earthshieldName then
      hasShield = true;
    elseif name == clearcastName then
      ccTexture = texture;
    end
  end

  if true or ccTexture then
    cooldownFrame1.texture:SetTexture(ccTexture);
  else
    cooldownFrame1.texture:SetTexture(0, 1, 0, .5);
  end

  if inCombat and canCastSpell(feralSpiritName, nextCastTime) then
    cooldownFrame2.texture:SetTexture(GetSpellTexture(feralSpiritName));
  else
    cooldownFrame2.texture:SetTexture(nil);
  end

  if (not inCombat) and (not hasShield) then
    addSpell(lightningshieldName);
  end

  local mana = UnitPower('player');
  local maxMana = UnitPowerMax('player');

  if hostile then
    local ranged = IsSpellInRange(stormstrikeName, 'target') == 0;
    local earthshockCastable = canCastSpell(earthshockName, nextCastTime);
    local stormstrikeCastable = canCastSpell(stormstrikeName, nextCastTime);
    local lavalashCastable = canCastSpell(lavalashName, nextCastTime);
    local maelstromFirst = maelstrom and (hasSSDebuff or ranged);

		local maelstromSpell = lightningBoltName;
		if maelstrom then
			if canCastSpell(chainlightningName, nextCastTime) then
				maelstromSpell = chainlightningName;
			end
		end

    if maelstromFirst then 
      addSpell(maelstromSpell);
    end

    if inCombat and canCastSpell(shamanisticRageName, nextCastTime) and (mana < .2 * maxMana) then
      local targetHealth = UnitHealth('target');
      if targetHealth and targetHealth == 1 then
        local unitName = UnitName('target');
        if unitName and unitName:find('Тренировочный манекен') then
          targetHealth = 99999;
        end
      end
      if targetHealth and targetHealth > 5000 then
        addSpell(shamanisticRageName);
      end
    end

    if ranged then
      if earthshockCastable then
        addSpell(earthshockName);
      elseif (not stormstrikeCastable) and (not lavalashCastable) then
        addSpell(autoattackName);
      end
    end

    if stormstrikeCastable then
      addSpell(stormstrikeName);
    end

    if maelstrom and (not maelstromFirst) then
      addSpell(maelstromSpell);
    end
    
    if (not ranged) and earthshockCastable then
      addSpell(earthshockName);
    end
    
    if lavalashCastable then
      addSpell(lavalashName);
    end

  else

    local health = UnitHealth('player');
    local maxHealth = UnitHealthMax('player');
  
    if health < (.6 * maxHealth) then
      addSpell(healingWaveName);
    elseif (not inCombat) and (mana >= (.90 * maxMana) or ccTexture) and health < (.75 * maxHealth) then
      addSpell(healingWaveName); -- heal yoself foo!
    end

  end

  if inCombat and not hasShield then
    addSpell(lightningshieldName);
  end
end

local function RecalcSpell()
	if (not inCombat) then
	  ShamWow_Enhance:RecalcMode(); -- check if you switched spec
	end
  AlertCheck();
  spellQueue = {};
  local startTime, duration = GetSpellCooldown(cooldownSpellName);
	local curTime = GetTime();
  local nextCastTime = curTime;
  if duration > 0 then
    local gcdEndTime = startTime + duration;
    if gcdEndTime > curTime then
      nextCastTime = gcdEndTime;
    end
    cooldownFrame:SetCooldown(startTime, duration);
    cooldownFrame:SetAlpha(1);
  	if mode == ELEM_MODE then
  	  -- we don't want to recalc too close to the last cast
  		if curTime - startTime < .5 then
  		  return;
  		end
  	end
  else
    cooldownFrame:SetAlpha(0);
  end
	local _, _, _, _, _, spellEndTime = UnitCastingInfo('player');
	if spellEndTime then
	  nextCastTime = math.min(nextCastTime, spellEndTime / 1000);
	end

	--local possessed = GetActionTexture(121);
	--if possessed ~= nil then
	-- do nothing
	if mode == RESTO_MODE then
    RecalcRestoQueue(nextCastTime);
	elseif mode == ELEM_MODE then
	  RecalcElemQueue(nextCastTime);
  else
    RecalcEnhanceQueue(nextCastTime);
  end
  for i=maxQueueSize,1,-1 do
    local spell = spellQueue[i];
    local f = spellQueueFrames[i];
    if spell then
      f.spellTexture:SetTexture(GetSpellTexture(spellQueue[i]));
      FrameStatusCheck(f,spell);
    else
      f.spellTexture:SetTexture(nil);
      f.overlayTexture:SetTexture(nil);
    end
  end
  local timeTextStr = '';
  if inCombat or hostile then
    timeLeft = ShamWow_Enhance.db.char.updateFrequency;
  else
    timeLeft = .5;
  end
  if #spellQueue == 0 then
    if (not possessed) and hostile and mode == ENHANCE_MODE then
      local castInfo = soonestCooldown(stormstrikeName, earthshockName, lavalashName);
      local timeUntilCast = castInfo.castTime - GetTime();
      if timeUntilCast < timeLeft then
        timeLeft = timeUntilCast;
      end
      if timeUntilCast < 1 then
        local f = spellQueueFrames[1];
        f.overlayTexture:SetTexture(.5, .5, .5, .5);
        f.spellTexture:SetTexture(GetSpellTexture(castInfo.spell));
        timeTextStr = string.format("%0.1f", timeUntilCast);
      else
        cooldownFrame:SetAlpha(0);
      end
    else
      cooldownFrame:SetAlpha(0);
    end
  end
  timeText:SetText(timeTextStr);
end

function ShamWow_Enhance:RecalcMode(...)
	local newMode = ShamWow_Enhance.db.char.mode;
	if (newMode == AUTO_MODE) then
	   if (GetSpellCooldown(totemOfWrathName) ~= nil) then
		   newMode = ELEM_MODE;
		 elseif (GetSpellCooldown(manatideName) ~= nil) then
		   newMode = RESTO_MODE;
		 else
		   newMode = ENHANCE_MODE;
		 end
	end
	if mode ~= newMode then
	  local modeName;
		if (newMode == ELEM_MODE) then
		  modeName = 'Стихии';
		elseif (newMode == RESTO_MODE) then
		  modeName = 'Исцеление';
		else
		  modeName = 'Совершенств';
		end
		mode = newMode;
	  swPrint('режим сменён на ' .. modeName);
	end
end

function ShamWow_Enhance:PLAYER_REGEN_ENABLED(...)
  inCombat = false;
end

function ShamWow_Enhance:PLAYER_REGEN_DISABLED(...)
  inCombat = true;
  ShamWow_Enhance:PLAYER_TARGET_CHANGED(); -- recompute if target is friend when combat starts
end

function ShamWow_Enhance:PLAYER_TARGET_CHANGED(...) 
  hostile = UnitName("target") and UnitCanAttack("player","target") and UnitHealth("target") > 0;
  timeLeft = 0;
end

function ShamWow_Enhance:OnEnable()
  local playerClass, englishClass = UnitClass("player");
  if englishClass ~= 'SHAMAN' then
    swPrint('ВЫ не шаман. Пожалуйста отключите для этого персонажа данный аддон ShamWow_Enhance');
  else
    playerId = UnitGUID("player");

    self:PARTY_MEMBERS_CHANGED(); -- recheck party members
    self:PLAYER_TARGET_CHANGED(); -- check target
		self:RecalcMode();

    -- Register for Function Events
    self:UnregisterAllEvents();
    self:RegisterEvent("PLAYER_TARGET_CHANGED");
    self:RegisterEvent("PLAYER_REGEN_ENABLED");
    self:RegisterEvent("PLAYER_REGEN_DISABLED");
    self:RegisterEvent("PARTY_MEMBERS_CHANGED");
    mainFrame:SetScript('OnUpdate', function(self, timeSinceLast)
      timeLeft = timeLeft - timeSinceLast;
      if timeLeft <= 0 then
        RecalcSpell();
      end
    end);
    swPrint('Включен');
  end
end

function ShamWow_Enhance:OnDisable()
  mainFrame:SetScript('OnUpdate', nil);
  swPrint(' отключен');
end

function ShamWow_Enhance:GetProperty(info)
  local propName = info[#info];
  local value = self.db.char[propName];
  return value;
end

function ShamWow_Enhance:SetProperty(info, newValue)
  local propName = info[#info];
  self.db.char[propName] = newValue;
	if propName == 'spacing' or propName == 'size' then
	  RepositionFrames();
	end
  if mainFrame and propName == 'size' then
    mainFrame:SetWidth(self.db.char.size);
    mainFrame:SetHeight(self.db.char.size);
  end 
end

function ShamWow_Enhance:GetOptions()
  local options = { 
    name    = "ShamWow_Enhance",
    handler   = ShamWow_Enhance,
    type    = 'group',
    childGroups ='tree',
    args  = {
      locked = {
        type = "toggle",
        name = "Фиксировать",
        get = "GetProperty",
        set = "SetProperty",
        order = 0,
      },
      mode = {
        type  = 'select',
        name  = "Mode",
        values = { 'Совершенств', 'Исцеление', 'Стихии', 'Авто' },
        get   = "GetProperty",
        set   = "SetProperty",
        order = 1,
      },
      size = {
        type = "range",
        name = "Размер",
        desc = "Размер панельки",
        min = 1,
        max = 200,
        step = 1,
        get = "GetProperty",
        set = "SetProperty",
        order = 2,
      },
      updateFrequency = {
        type = 'range',
        name = "Частота обновления",
        desc = "Задержка в секундах перед проверкой следующего доспупного заклинания.",
        min = .05,
        max = .5,
        step = .05,
        get = "GetProperty",
        set = "SetProperty",
        order = 3,
      },
			spacing = {
        type = 'range',
        name = "Промежуток мконок",
        min = 1,
        max = 200,
        step = 1,
        get = "GetProperty",
        set = "SetProperty",
        order = 4,
			},
      threatThreshold = {
        type = 'range',
        name = "Угроза",
        desc = "Отображение угрозы, если ваш процент угрозы достигает данного значения. 100% означает, что на вас аггро! Если вы не хотите видеть индикатор угрозы то установите значение на 101.",
        min = 0,
        max = 101,
        step = 1,
        get = "GetProperty",
        set = "SetProperty",
        order = 5,
      }
    },
  }
  return options
end

